@@ -44,6 +44,9 @@
 
 #if defined __ANDROID__ || defined __linux__
 #if defined __ANDROID__
+#if __ANDROID_API__ >= 18
+#include <sys/auxv.h> // getauxval()
+#endif
 #include <dlfcn.h>
 #endif
 #include <stdint.h>
@@ -105,20 +108,20 @@ namespace ncnn {
 #if defined __ANDROID__
 // Probe the system's C library for a 'getauxval' function and call it if
 // it exits, or return 0 for failure. This function is available since API
-// level 20.
-//
-// This code does *NOT* check for '__ANDROID_API__ >= 20' to support the
-// edge case where some NDK developers use headers for a platform that is
-// newer than the one really targetted by their application.
-// This is typically done to use newer native APIs only when running on more
-// recent Android versions, and requires careful symbol management.
+// level 18.
 //
 // Note that getauxval() can't really be re-implemented here, because
 // its implementation does not parse /proc/self/auxv. Instead it depends
 // on values  that are passed by the kernel at process-init time to the
 // C runtime initialization layer.
-static unsigned int get_elf_hwcap_from_getauxval()
+static unsigned int get_elf_hwcap_from_getauxval(unsigned int type)
 {
+#if __ANDROID_API__ >= 18
+    unsigned int hwcap = getauxval(type);
+    if (hwcap)
+        return hwcap;
+#endif
+
     typedef unsigned long getauxval_func_t(unsigned long);
 
     dlerror();
@@ -138,7 +141,7 @@ static unsigned int get_elf_hwcap_from_getauxval()
     else
     {
         // Note: getauxval() returns 0 on failure. Doesn't touch errno.
-        result = (unsigned int)(*func)(AT_HWCAP);
+        result = (unsigned int)(*func)(type);
     }
     dlclose(libc_handle);
 
@@ -147,7 +150,7 @@ static unsigned int get_elf_hwcap_from_getauxval()
 #endif // defined __ANDROID__
 
 // extract the ELF HW capabilities bitmap from /proc/self/auxv
-static unsigned int get_elf_hwcap_from_proc_self_auxv()
+static unsigned int get_elf_hwcap_from_proc_self_auxv(unsigned int type)
 {
     FILE* fp = fopen("/proc/self/auxv", "rb");
     if (!fp)
@@ -156,7 +159,7 @@ static unsigned int get_elf_hwcap_from_proc_self_auxv()
         return 0;
     }
 
-#if __aarch64__ || __riscv_xlen == 64
+#if __aarch64__ || __mips64 || __riscv_xlen == 64
     struct
     {
         uint64_t tag;
@@ -181,7 +184,7 @@ static unsigned int get_elf_hwcap_from_proc_self_auxv()
         if (entry.tag == 0 && entry.value == 0)
             break;
 
-        if (entry.tag == AT_HWCAP)
+        if (entry.tag == type)
         {
             result = entry.value;
             break;
@@ -193,26 +196,36 @@ static unsigned int get_elf_hwcap_from_proc_self_auxv()
     return result;
 }
 
-static unsigned int get_elf_hwcap()
+static unsigned int get_elf_hwcap(unsigned int type)
 {
 #if defined __ANDROID__
-    unsigned int hwcap = get_elf_hwcap_from_getauxval();
+    unsigned int hwcap = get_elf_hwcap_from_getauxval(type);
     if (hwcap)
         return hwcap;
 #endif
 
-    return get_elf_hwcap_from_proc_self_auxv();
+    return get_elf_hwcap_from_proc_self_auxv(type);
 }
 
-static unsigned int g_hwcaps = get_elf_hwcap();
+static unsigned int g_hwcaps = get_elf_hwcap(AT_HWCAP);
+static unsigned int g_hwcaps2 = get_elf_hwcap(AT_HWCAP2);
 
 #if __aarch64__
 // from arch/arm64/include/uapi/asm/hwcap.h
-#define HWCAP_ASIMD   (1 << 1)
-#define HWCAP_ASIMDHP (1 << 10)
-#define HWCAP_ASIMDDP (1 << 20)
+#define HWCAP_ASIMD     (1 << 1)
+#define HWCAP_ASIMDHP   (1 << 10)
+#define HWCAP_ASIMDDP   (1 << 20)
+#define HWCAP_SVE       (1 << 22)
+#define HWCAP_ASIMDFHM  (1 << 23)
+#define HWCAP2_SVE2     (1 << 1)
+#define HWCAP2_SVEI8MM  (1 << 9)
+#define HWCAP2_SVEF32MM (1 << 10)
+#define HWCAP2_SVEBF16  (1 << 12)
+#define HWCAP2_I8MM     (1 << 13)
+#define HWCAP2_BF16     (1 << 14)
 #else
 // from arch/arm/include/uapi/asm/hwcap.h
+#define HWCAP_EDSP  (1 << 7)
 #define HWCAP_NEON  (1 << 12)
 #define HWCAP_VFPv4 (1 << 16)
 #endif
@@ -363,6 +376,25 @@ int CpuSet::num_enabled() const
 }
 #endif
 
+int cpu_support_arm_edsp()
+{
+#if defined __ANDROID__ || defined __linux__
+#if __aarch64__
+    return 0;
+#else
+    return g_hwcaps & HWCAP_EDSP;
+#endif
+#elif __APPLE__
+#if __aarch64__
+    return 0;
+#else
+    return g_hw_cputype == CPU_TYPE_ARM;
+#endif
+#else
+    return 0;
+#endif
+}
+
 int cpu_support_arm_neon()
 {
 #if defined __ANDROID__ || defined __linux__
@@ -440,6 +472,158 @@ int cpu_support_arm_asimddp()
 #endif
 }
 
+int cpu_support_arm_asimdfhm()
+{
+#if defined __ANDROID__ || defined __linux__
+#if __aarch64__
+    return g_hwcaps & HWCAP_ASIMDFHM;
+#else
+    return 0;
+#endif
+#elif __APPLE__
+#if __aarch64__
+    return g_hw_cpufamily == CPUFAMILY_ARM_LIGHTNING_THUNDER || g_hw_cpufamily == CPUFAMILY_ARM_FIRESTORM_ICESTORM || g_hw_cpufamily == CPUFAMILY_ARM_AVALANCHE_BLIZZARD;
+#else
+    return 0;
+#endif
+#else
+    return 0;
+#endif
+}
+
+int cpu_support_arm_bf16()
+{
+#if defined __ANDROID__ || defined __linux__
+#if __aarch64__
+    return g_hwcaps2 & HWCAP2_BF16;
+#else
+    return 0;
+#endif
+#elif __APPLE__
+#if __aarch64__
+    return 0; // no known apple cpu support armv8.6 bf16
+#else
+    return 0;
+#endif
+#else
+    return 0;
+#endif
+}
+
+int cpu_support_arm_i8mm()
+{
+#if defined __ANDROID__ || defined __linux__
+#if __aarch64__
+    return g_hwcaps2 & HWCAP2_I8MM;
+#else
+    return 0;
+#endif
+#elif __APPLE__
+#if __aarch64__
+    return 0; // no known apple cpu support armv8.6 i8mm
+#else
+    return 0;
+#endif
+#else
+    return 0;
+#endif
+}
+
+int cpu_support_arm_sve()
+{
+#if defined __ANDROID__ || defined __linux__
+#if __aarch64__
+    return g_hwcaps & HWCAP_SVE;
+#else
+    return 0;
+#endif
+#elif __APPLE__
+#if __aarch64__
+    return 0; // no known apple cpu support armv8.6 sve
+#else
+    return 0;
+#endif
+#else
+    return 0;
+#endif
+}
+
+int cpu_support_arm_sve2()
+{
+#if defined __ANDROID__ || defined __linux__
+#if __aarch64__
+    return g_hwcaps2 & HWCAP2_SVE2;
+#else
+    return 0;
+#endif
+#elif __APPLE__
+#if __aarch64__
+    return 0; // no known apple cpu support armv8.6 sve2
+#else
+    return 0;
+#endif
+#else
+    return 0;
+#endif
+}
+
+int cpu_support_arm_svebf16()
+{
+#if defined __ANDROID__ || defined __linux__
+#if __aarch64__
+    return g_hwcaps2 & HWCAP2_SVEBF16;
+#else
+    return 0;
+#endif
+#elif __APPLE__
+#if __aarch64__
+    return 0; // no known apple cpu support armv8.6 svebf16
+#else
+    return 0;
+#endif
+#else
+    return 0;
+#endif
+}
+
+int cpu_support_arm_svei8mm()
+{
+#if defined __ANDROID__ || defined __linux__
+#if __aarch64__
+    return g_hwcaps2 & HWCAP2_SVEI8MM;
+#else
+    return 0;
+#endif
+#elif __APPLE__
+#if __aarch64__
+    return 0; // no known apple cpu support armv8.6 svei8mm
+#else
+    return 0;
+#endif
+#else
+    return 0;
+#endif
+}
+
+int cpu_support_arm_svef32mm()
+{
+#if defined __ANDROID__ || defined __linux__
+#if __aarch64__
+    return g_hwcaps2 & HWCAP2_SVEF32MM;
+#else
+    return 0;
+#endif
+#elif __APPLE__
+#if __aarch64__
+    return 0; // no known apple cpu support armv8.6 svef32mm
+#else
+    return 0;
+#endif
+#else
+    return 0;
+#endif
+}
+
 #if defined(__i386__) || defined(__x86_64__) || defined(_M_IX86) || defined(_M_X64)
 static inline void x86_cpuid(int level, unsigned int out[4])
 {
@@ -552,6 +736,23 @@ static int get_cpu_support_x86_xop()
     return cpu_info[2] & (1u << 11);
 }
 
+static int get_cpu_support_x86_f16c()
+{
+#if !NCNN_F16C
+    return 0;
+#endif
+    unsigned int cpu_info[4] = {0};
+    x86_cpuid(0, cpu_info);
+
+    int nIds = cpu_info[0];
+    if (nIds < 1)
+        return 0;
+
+    x86_cpuid(1, cpu_info);
+
+    return cpu_info[2] & (1u << 29);
+}
+
 static int get_cpu_support_x86_avx2()
 {
 #if !NCNN_AVX2
@@ -628,7 +829,7 @@ static int get_cpu_support_x86_avx512()
         return 0;
 
     x86_cpuid_sublevel(7, 0, cpu_info);
-    return (cpu_info[1] & (1u << 16)) && (cpu_info[1] & (1u << 31));
+    return (cpu_info[1] & (1u << 16)) && (cpu_info[1] & (1u << 17)) && (cpu_info[1] & (1u << 28)) && (cpu_info[1] & (1u << 30)) && (cpu_info[1] & (1u << 31));
 }
 
 static int get_cpu_support_x86_avx512_vnni()
@@ -660,21 +861,81 @@ static int get_cpu_support_x86_avx512_vnni()
     return cpu_info[2] & (1u << 11);
 }
 
+static int get_cpu_support_x86_avx512_bf16()
+{
+#if !NCNN_AVX512BF16
+    return 0;
+#endif
+    unsigned int cpu_info[4] = {0};
+    x86_cpuid(0, cpu_info);
+
+    int nIds = cpu_info[0];
+    if (nIds < 7)
+        return 0;
+
+    x86_cpuid(1, cpu_info);
+    // check AVX XSAVE OSXSAVE
+    if (!(cpu_info[2] & (1u << 28)) || !(cpu_info[2] & (1u << 26)) || !(cpu_info[2] & (1u << 27)))
+        return 0;
+
+    // check XSAVE enabled by kernel
+    if ((x86_get_xcr0() & 6) != 6)
+        return 0;
+
+    x86_cpuid_sublevel(7, 1, cpu_info);
+    return cpu_info[0] & (1u << 5);
+}
+
+static int get_cpu_support_x86_avx512_fp16()
+{
+#if !NCNN_AVX512FP16
+    return 0;
+#endif
+    unsigned int cpu_info[4] = {0};
+    x86_cpuid(0, cpu_info);
+
+    int nIds = cpu_info[0];
+    if (nIds < 7)
+        return 0;
+
+    x86_cpuid(1, cpu_info);
+    // check AVX XSAVE OSXSAVE
+    if (!(cpu_info[2] & (1u << 28)) || !(cpu_info[2] & (1u << 26)) || !(cpu_info[2] & (1u << 27)))
+        return 0;
+
+    // check XSAVE enabled by kernel
+    if ((x86_get_xcr0() & 6) != 6)
+        return 0;
+
+    // check avx512 XSAVE enabled by kernel
+    if ((x86_get_xcr0() & 0xe0) != 0xe0)
+        return 0;
+
+    x86_cpuid_sublevel(7, 0, cpu_info);
+    return cpu_info[3] & (1u << 23);
+}
+
 static int g_cpu_support_x86_avx = get_cpu_support_x86_avx();
 static int g_cpu_support_x86_fma = get_cpu_support_x86_fma();
 static int g_cpu_support_x86_xop = get_cpu_support_x86_xop();
+static int g_cpu_support_x86_f16c = get_cpu_support_x86_f16c();
 static int g_cpu_support_x86_avx2 = get_cpu_support_x86_avx2();
 static int g_cpu_support_x86_avx_vnni = get_cpu_support_x86_avx_vnni();
 static int g_cpu_support_x86_avx512 = get_cpu_support_x86_avx512();
 static int g_cpu_support_x86_avx512_vnni = get_cpu_support_x86_avx512_vnni();
+static int g_cpu_support_x86_avx512_bf16 = get_cpu_support_x86_avx512_bf16();
+static int g_cpu_support_x86_avx512_fp16 = get_cpu_support_x86_avx512_fp16();
 #else  // defined(__i386__) || defined(__x86_64__) || defined(_M_IX86) || defined(_M_X64)
 static const int g_cpu_support_x86_avx = 0;
 static const int g_cpu_support_x86_fma = 0;
 static const int g_cpu_support_x86_xop = 0;
+static const int g_cpu_support_x86_f16c = 0;
 static const int g_cpu_support_x86_avx2 = 0;
 static const int g_cpu_support_x86_avx_vnni = 0;
 static const int g_cpu_support_x86_avx512 = 0;
 static const int g_cpu_support_x86_avx512_vnni = 0;
+static const int g_cpu_support_x86_avx512_bf16 = 0;
+static const int g_cpu_support_x86_avx512_fp16 = 0;
 #endif // defined(__i386__) || defined(__x86_64__) || defined(_M_IX86) || defined(_M_X64)
 
 int cpu_support_x86_avx()
@@ -692,6 +953,11 @@ int cpu_support_x86_xop()
     return g_cpu_support_x86_xop;
 }
 
+int cpu_support_x86_f16c()
+{
+    return g_cpu_support_x86_f16c;
+}
+
 int cpu_support_x86_avx2()
 {
     return g_cpu_support_x86_avx2;
@@ -712,6 +978,16 @@ int cpu_support_x86_avx512_vnni()
     return g_cpu_support_x86_avx512_vnni;
 }
 
+int cpu_support_x86_avx512_bf16()
+{
+    return g_cpu_support_x86_avx512_bf16;
+}
+
+int cpu_support_x86_avx512_fp16()
+{
+    return g_cpu_support_x86_avx512_fp16;
+}
+
 int cpu_support_mips_msa()
 {
 #if defined __ANDROID__ || defined __linux__
