@@ -50,7 +50,7 @@ Layer::Layer()
     support_image_storage = false;
     support_tensor_storage = false;
 
-    support_weight_fp16_storage = false;
+    support_reserved_00 = false;
 
     typeindex = -1;
 
@@ -253,20 +253,6 @@ Layer* create_layer(int index)
     }
     else
 #endif // NCNN_RUNTIME_CPU && NCNN_AVX
-#if NCNN_RUNTIME_CPU && NCNN_ARM82
-    if (ncnn::cpu_support_arm_asimdhp())
-    {
-        layer_creator = layer_registry_arm82[index].creator;
-    }
-    else
-#endif // NCNN_RUNTIME_CPU && NCNN_ARM82
-#if NCNN_RUNTIME_CPU && NCNN_MMI
-    if (ncnn::cpu_support_mips_msa() && ncnn::cpu_support_loongson_mmi())
-    {
-        layer_creator = layer_registry_mmi[index].creator;
-    }
-    else
-#endif // NCNN_RUNTIME_CPU && NCNN_MMI
 #if NCNN_RUNTIME_CPU && NCNN_MSA
     if (ncnn::cpu_support_mips_msa())
     {
