@@ -15,11 +15,7 @@
 #include "gru_riscv.h"
 
 #if __riscv_vector
-#ifdef RVV_SPEC_0_7
-#include "riscv_v_071_fix.h"
-#else
 #include <riscv_vector.h>
-#endif
 #endif // __riscv_vector
 
 namespace ncnn {
@@ -44,6 +40,7 @@ static int gru(const Mat& bottom_blob, Mat& top_blob, int reverse, const Mat& we
         int ti = reverse ? T - 1 - t : t;
 
         const float* x = bottom_blob.row(ti);
+        #pragma omp parallel for num_threads(opt.num_threads)
         for (int q = 0; q < num_output; q++)
         {
             float* gates_data = gates.row(q);
@@ -75,8 +72,8 @@ static int gru(const Mat& bottom_blob, Mat& top_blob, int reverse, const Mat& we
 
                 _xcr = vfmul_vv_f32m8(_xcr, _x, vl);
                 _xcu = vfmul_vv_f32m8(_xcu, _x, vl);
-                _scalar_r = vfredsum_vs_f32m8_f32m1(_scalar_r, _xcr, _scalar_r, vl);
-                _scalar_u = vfredsum_vs_f32m8_f32m1(_scalar_u, _xcu, _scalar_u, vl);
+                _scalar_r = vfredusum_vs_f32m8_f32m1(_scalar_r, _xcr, _scalar_r, vl);
+                _scalar_u = vfredusum_vs_f32m8_f32m1(_scalar_u, _xcu, _scalar_u, vl);
 
                 R = vfmv_f_s_f32m1_f32(_scalar_r);
                 U = vfmv_f_s_f32m1_f32(_scalar_u);
@@ -105,8 +102,8 @@ static int gru(const Mat& bottom_blob, Mat& top_blob, int reverse, const Mat& we
 
                 _hcr = vfmul_vv_f32m8(_hcr, _h_cont, vl);
                 _hcu = vfmul_vv_f32m8(_hcu, _h_cont, vl);
-                _scalar_r = vfredsum_vs_f32m8_f32m1(_scalar_r, _hcr, _scalar_r, vl);
-                _scalar_u = vfredsum_vs_f32m8_f32m1(_scalar_u, _hcu, _scalar_u, vl);
+                _scalar_r = vfredusum_vs_f32m8_f32m1(_scalar_r, _hcr, _scalar_r, vl);
+                _scalar_u = vfredusum_vs_f32m8_f32m1(_scalar_u, _hcu, _scalar_u, vl);
 
                 R = vfmv_f_s_f32m1_f32(_scalar_r);
                 U = vfmv_f_s_f32m1_f32(_scalar_u);
@@ -146,7 +143,7 @@ static int gru(const Mat& bottom_blob, Mat& top_blob, int reverse, const Mat& we
                 vfloat32m1_t _scalar_n = vfmv_s_f_f32m1(vundefined_f32m1(), N, vl);
 
                 _h_cont = vfmul_vv_f32m8(_whc_n, _h_cont, vl);
-                _scalar_n = vfredsum_vs_f32m8_f32m1(_scalar_n, _h_cont, _scalar_n, vl);
+                _scalar_n = vfredusum_vs_f32m8_f32m1(_scalar_n, _h_cont, _scalar_n, vl);
 
                 N = vfmv_f_s_f32m1_f32(_scalar_n);
                 n_out2 -= vl;
@@ -170,7 +167,7 @@ static int gru(const Mat& bottom_blob, Mat& top_blob, int reverse, const Mat& we
                 vfloat32m1_t _scalar_n = vfmv_s_f_f32m1(vundefined_f32m1(), N, vl);
 
                 _xcn = vfmul_vv_f32m8(_x, _xcn, vl);
-                _scalar_n = vfredsum_vs_f32m8_f32m1(_scalar_n, _xcn, _scalar_n, vl);
+                _scalar_n = vfredusum_vs_f32m8_f32m1(_scalar_n, _xcn, _scalar_n, vl);
                 N = vfmv_f_s_f32m1_f32(_scalar_n);
 
                 n2 -= vl;
@@ -189,6 +186,7 @@ static int gru(const Mat& bottom_blob, Mat& top_blob, int reverse, const Mat& we
 
         // h_t := (1 - update) .* new + update .* h_{t-1}
         float* output_data = top_blob.row(ti);
+        #pragma omp parallel for num_threads(opt.num_threads)
         for (int q = 0; q < num_output; q++)
         {
             const float* gates_data = gates.row(q);
@@ -407,6 +405,7 @@ static int gru_fp16s(const Mat& bottom_blob, Mat& top_blob, int reverse, const M
         int ti = reverse ? T - 1 - t : t;
 
         const __fp16* x = bottom_blob.row<const __fp16>(ti);
+        #pragma omp parallel for num_threads(opt.num_threads)
         for (int q = 0; q < num_output; q++)
         {
             float* gates_data = gates.row(q);
@@ -438,8 +437,8 @@ static int gru_fp16s(const Mat& bottom_blob, Mat& top_blob, int reverse, const M
 
                 _xcr = vfmul_vv_f32m8(_xcr, _x, vl);
                 _xcu = vfmul_vv_f32m8(_xcu, _x, vl);
-                _scalar_r = vfredsum_vs_f32m8_f32m1(_scalar_r, _xcr, _scalar_r, vl);
-                _scalar_u = vfredsum_vs_f32m8_f32m1(_scalar_u, _xcu, _scalar_u, vl);
+                _scalar_r = vfredusum_vs_f32m8_f32m1(_scalar_r, _xcr, _scalar_r, vl);
+                _scalar_u = vfredusum_vs_f32m8_f32m1(_scalar_u, _xcu, _scalar_u, vl);
 
                 R = vfmv_f_s_f32m1_f32(_scalar_r);
                 U = vfmv_f_s_f32m1_f32(_scalar_u);
@@ -468,8 +467,8 @@ static int gru_fp16s(const Mat& bottom_blob, Mat& top_blob, int reverse, const M
 
                 _hcr = vfmul_vv_f32m8(_hcr, _h_cont, vl);
                 _hcu = vfmul_vv_f32m8(_hcu, _h_cont, vl);
-                _scalar_r = vfredsum_vs_f32m8_f32m1(_scalar_r, _hcr, _scalar_r, vl);
-                _scalar_u = vfredsum_vs_f32m8_f32m1(_scalar_u, _hcu, _scalar_u, vl);
+                _scalar_r = vfredusum_vs_f32m8_f32m1(_scalar_r, _hcr, _scalar_r, vl);
+                _scalar_u = vfredusum_vs_f32m8_f32m1(_scalar_u, _hcu, _scalar_u, vl);
 
                 R = vfmv_f_s_f32m1_f32(_scalar_r);
                 U = vfmv_f_s_f32m1_f32(_scalar_u);
@@ -509,7 +508,7 @@ static int gru_fp16s(const Mat& bottom_blob, Mat& top_blob, int reverse, const M
                 vfloat32m1_t _scalar_n = vfmv_s_f_f32m1(vundefined_f32m1(), N, vl);
 
                 _h_cont = vfmul_vv_f32m8(_whc_n, _h_cont, vl);
-                _scalar_n = vfredsum_vs_f32m8_f32m1(_scalar_n, _h_cont, _scalar_n, vl);
+                _scalar_n = vfredusum_vs_f32m8_f32m1(_scalar_n, _h_cont, _scalar_n, vl);
 
                 N = vfmv_f_s_f32m1_f32(_scalar_n);
                 n_out2 -= vl;
@@ -533,7 +532,7 @@ static int gru_fp16s(const Mat& bottom_blob, Mat& top_blob, int reverse, const M
                 vfloat32m1_t _scalar_n = vfmv_s_f_f32m1(vundefined_f32m1(), N, vl);
 
                 _xcn = vfmul_vv_f32m8(_x, _xcn, vl);
-                _scalar_n = vfredsum_vs_f32m8_f32m1(_scalar_n, _xcn, _scalar_n, vl);
+                _scalar_n = vfredusum_vs_f32m8_f32m1(_scalar_n, _xcn, _scalar_n, vl);
                 N = vfmv_f_s_f32m1_f32(_scalar_n);
 
                 n2 -= vl;
@@ -552,6 +551,7 @@ static int gru_fp16s(const Mat& bottom_blob, Mat& top_blob, int reverse, const M
 
         // h_t := (1 - update) .* new + update .* h_{t-1}
         __fp16* output_data = top_blob.row<__fp16>(ti);
+        #pragma omp parallel for num_threads(opt.num_threads)
         for (int q = 0; q < num_output; q++)
         {
             const float* gates_data = gates.row(q);
@@ -735,6 +735,7 @@ static int gru_fp16sa(const Mat& bottom_blob, Mat& top_blob, int reverse, const
         int ti = reverse ? T - 1 - t : t;
 
         const __fp16* x = bottom_blob.row<const __fp16>(ti);
+        #pragma omp parallel for num_threads(opt.num_threads)
         for (int q = 0; q < num_output; q++)
         {
             float* gates_data = gates.row(q);
@@ -766,8 +767,8 @@ static int gru_fp16sa(const Mat& bottom_blob, Mat& top_blob, int reverse, const
 
                 _xcr = vfmul_vv_f16m8(_xcr, _x, vl);
                 _xcu = vfmul_vv_f16m8(_xcu, _x, vl);
-                _scalar_r = vfredsum_vs_f16m8_f16m1(_scalar_r, _xcr, _scalar_r, vl);
-                _scalar_u = vfredsum_vs_f16m8_f16m1(_scalar_u, _xcu, _scalar_u, vl);
+                _scalar_r = vfredusum_vs_f16m8_f16m1(_scalar_r, _xcr, _scalar_r, vl);
+                _scalar_u = vfredusum_vs_f16m8_f16m1(_scalar_u, _xcu, _scalar_u, vl);
 
                 R = vfmv_f_s_f16m1_f16(_scalar_r);
                 U = vfmv_f_s_f16m1_f16(_scalar_u);
@@ -793,8 +794,8 @@ static int gru_fp16sa(const Mat& bottom_blob, Mat& top_blob, int reverse, const
 
                 _hcr = vfmul_vv_f16m4(_hcr, _h_cont, vl);
                 _hcu = vfmul_vv_f16m4(_hcu, _h_cont, vl);
-                _scalar_r = vfredsum_vs_f16m4_f16m1(_scalar_r, _hcr, _scalar_r, vl);
-                _scalar_u = vfredsum_vs_f16m4_f16m1(_scalar_u, _hcu, _scalar_u, vl);
+                _scalar_r = vfredusum_vs_f16m4_f16m1(_scalar_r, _hcr, _scalar_r, vl);
+                _scalar_u = vfredusum_vs_f16m4_f16m1(_scalar_u, _hcu, _scalar_u, vl);
 
                 R = vfmv_f_s_f16m1_f16(_scalar_r);
                 U = vfmv_f_s_f16m1_f16(_scalar_u);
@@ -831,7 +832,7 @@ static int gru_fp16sa(const Mat& bottom_blob, Mat& top_blob, int reverse, const
                 vfloat16m1_t _scalar_n = vfmv_s_f_f16m1(vundefined_f16m1(), N, vl);
 
                 _h_cont = vfmul_vv_f16m4(_whc_n, _h_cont, vl);
-                _scalar_n = vfredsum_vs_f16m4_f16m1(_scalar_n, _h_cont, _scalar_n, vl);
+                _scalar_n = vfredusum_vs_f16m4_f16m1(_scalar_n, _h_cont, _scalar_n, vl);
 
                 N = vfmv_f_s_f16m1_f16(_scalar_n);
                 n_out2 -= vl;
@@ -852,7 +853,7 @@ static int gru_fp16sa(const Mat& bottom_blob, Mat& top_blob, int reverse, const
                 vfloat16m1_t _scalar_n = vfmv_s_f_f16m1(vundefined_f16m1(), N, vl);
 
                 _xcn = vfmul_vv_f16m8(_x, _xcn, vl);
-                _scalar_n = vfredsum_vs_f16m8_f16m1(_scalar_n, _xcn, _scalar_n, vl);
+                _scalar_n = vfredusum_vs_f16m8_f16m1(_scalar_n, _xcn, _scalar_n, vl);
                 N = vfmv_f_s_f16m1_f16(_scalar_n);
 
                 n2 -= vl;
@@ -869,6 +870,7 @@ static int gru_fp16sa(const Mat& bottom_blob, Mat& top_blob, int reverse, const
 
         // h_t := (1 - update) .* new + update .* h_{t-1}
         __fp16* output_data = top_blob.row<__fp16>(ti);
+        #pragma omp parallel for num_threads(opt.num_threads)
         for (int q = 0; q < num_output; q++)
         {
             const float* gates_data = gates.row(q);
