@@ -20,18 +20,17 @@
 #if __ARM_NEON
 #include <arm_neon.h>
 #endif
+#if __SSE2__
+#include <emmintrin.h>
 #if __AVX__
 #include <immintrin.h>
 #endif
+#endif
 #if __mips_msa
 #include <msa.h>
 #endif
 #if __riscv_vector
-#ifdef RVV_SPEC_0_7
-#include "layer/riscv/riscv_v_071_fix.h"
-#else
 #include <riscv_vector.h>
-#endif
 #include "cpu.h" // cpu_riscv_vlenb()
 #endif
 
@@ -116,10 +115,16 @@ public:
     void fill(float16x8_t _v);
 #endif // __ARM_FEATURE_FP16_VECTOR_ARITHMETIC
 #endif // __ARM_NEON
+#if __SSE2__
 #if __AVX__
-    void fill(__m256 _v);
-    void fill(__m128i _v);
+#if __AVX512F__
+    void fill(__m512 _v);
+#endif // __AVX512F__
+    void fill(__m256 _v, int i = 0);
 #endif // __AVX__
+    void fill(__m128 _v);
+    void fill(__m128i _v);
+#endif // __SSE2__
 #if __mips_msa
     void fill(v4f32 _v);
 #endif // __mips_msa
@@ -750,27 +755,19 @@ NCNN_EXPORT NCNN_FORCEINLINE float bfloat16_to_float32(unsigned short value)
     tmp.u = value << 16;
     return tmp.f;
 }
-#if __ARM_NEON
-NCNN_EXPORT NCNN_FORCEINLINE uint16x4_t vcvt_bf16_f32(float32x4_t _v)
-{
-    return vshrn_n_u32(vreinterpretq_u32_f32(_v), 16);
-}
-NCNN_EXPORT NCNN_FORCEINLINE float32x4_t vcvt_f32_bf16(uint16x4_t _v)
-{
-    return vreinterpretq_f32_u32(vshll_n_u16(_v, 16));
-}
-#endif // __ARM_NEON
 
 // mat process
 enum BorderType
 {
     BORDER_CONSTANT = 0,
     BORDER_REPLICATE = 1,
+    BORDER_REFLECT = 2,
     BORDER_TRANSPARENT = -233,
 };
 NCNN_EXPORT void copy_make_border(const Mat& src, Mat& dst, int top, int bottom, int left, int right, int type, float v, const Option& opt = Option());
 NCNN_EXPORT void copy_make_border_3d(const Mat& src, Mat& dst, int top, int bottom, int left, int right, int front, int behind, int type, float v, const Option& opt = Option());
 NCNN_EXPORT void copy_cut_border(const Mat& src, Mat& dst, int top, int bottom, int left, int right, const Option& opt = Option());
+NCNN_EXPORT void copy_cut_border_3d(const Mat& src, Mat& dst, int top, int bottom, int left, int right, int front, int behind, const Option& opt = Option());
 NCNN_EXPORT void resize_nearest(const Mat& src, Mat& dst, int w, int h, const Option& opt = Option());
 NCNN_EXPORT void resize_bilinear(const Mat& src, Mat& dst, int w, int h, const Option& opt = Option());
 NCNN_EXPORT void resize_bicubic(const Mat& src, Mat& dst, int w, int h, const Option& opt = Option());
@@ -902,48 +899,16 @@ NCNN_FORCEINLINE void Mat::fill(float _v)
     int size = (int)total();
     float* ptr = (float*)data;
 
-#if __ARM_NEON
-    int nn = size >> 2;
-    int remain = size - (nn << 2);
-#else
-    int remain = size;
-#endif // __ARM_NEON
-
+    int i = 0;
 #if __ARM_NEON
     float32x4_t _c = vdupq_n_f32(_v);
-#if __aarch64__
-    if (nn > 0)
+    for (; i + 3 < size; i += 4)
     {
-        asm volatile(
-            "0:                             \n"
-            "subs       %w0, %w0, #1        \n"
-            "st1        {%4.4s}, [%1], #16  \n"
-            "bne        0b                  \n"
-            : "=r"(nn), // %0
-            "=r"(ptr) // %1
-            : "0"(nn),
-            "1"(ptr),
-            "w"(_c) // %4
-            : "cc", "memory");
+        vst1q_f32(ptr, _c);
+        ptr += 4;
     }
-#else
-    if (nn > 0)
-    {
-        asm volatile(
-            "0:                             \n"
-            "subs       %0, #1              \n"
-            "vst1.f32   {%e4-%f4}, [%1 :128]!\n"
-            "bne        0b                  \n"
-            : "=r"(nn), // %0
-            "=r"(ptr) // %1
-            : "0"(nn),
-            "1"(ptr),
-            "w"(_c) // %4
-            : "cc", "memory");
-    }
-#endif // __aarch64__
 #endif // __ARM_NEON
-    for (; remain > 0; remain--)
+    for (; i < size; i++)
     {
         *ptr++ = _v;
     }
@@ -954,48 +919,16 @@ NCNN_FORCEINLINE void Mat::fill(int _v)
     int size = (int)total();
     int* ptr = (int*)data;
 
-#if __ARM_NEON
-    int nn = size >> 2;
-    int remain = size - (nn << 2);
-#else
-    int remain = size;
-#endif // __ARM_NEON
-
+    int i = 0;
 #if __ARM_NEON
     int32x4_t _c = vdupq_n_s32(_v);
-#if __aarch64__
-    if (nn > 0)
+    for (; i + 3 < size; i += 4)
     {
-        asm volatile(
-            "0:                             \n"
-            "subs       %w0, %w0, #1        \n"
-            "st1        {%4.4s}, [%1], #16  \n"
-            "bne        0b                  \n"
-            : "=r"(nn), // %0
-            "=r"(ptr) // %1
-            : "0"(nn),
-            "1"(ptr),
-            "w"(_c) // %4
-            : "cc", "memory");
+        vst1q_s32(ptr, _c);
+        ptr += 4;
     }
-#else
-    if (nn > 0)
-    {
-        asm volatile(
-            "0:                             \n"
-            "subs       %0, #1              \n"
-            "vst1.s32   {%e4-%f4}, [%1 :128]!\n"
-            "bne        0b                  \n"
-            : "=r"(nn), // %0
-            "=r"(ptr) // %1
-            : "0"(nn),
-            "1"(ptr),
-            "w"(_c) // %4
-            : "cc", "memory");
-    }
-#endif // __aarch64__
 #endif // __ARM_NEON
-    for (; remain > 0; remain--)
+    for (; i < size; i++)
     {
         *ptr++ = _v;
     }
@@ -1070,9 +1003,26 @@ NCNN_FORCEINLINE void Mat::fill(float16x8_t _v)
 }
 #endif // __ARM_FEATURE_FP16_VECTOR_ARITHMETIC
 #endif // __ARM_NEON
+
+#if __SSE2__
 #if __AVX__
-NCNN_FORCEINLINE void Mat::fill(__m256 _v)
+#if __AVX512F__
+NCNN_FORCEINLINE void Mat::fill(__m512 _v)
 {
+    int size = (int)total();
+    float* ptr = (float*)data;
+    for (int i = 0; i < size; i++)
+    {
+        _mm512_storeu_ps(ptr, _v);
+        ptr += 16;
+    }
+}
+#endif // __AVX512F__
+NCNN_FORCEINLINE void Mat::fill(__m256 _v, int _i)
+{
+    // old gcc cannot overload __m128 and __m256 type
+    // add a dummy int parameter for different mangled function symbol
+    (void)_i;
     int size = (int)total();
     float* ptr = (float*)data;
     for (int i = 0; i < size; i++)
@@ -1081,6 +1031,17 @@ NCNN_FORCEINLINE void Mat::fill(__m256 _v)
         ptr += 8;
     }
 }
+#endif // __AVX__
+NCNN_FORCEINLINE void Mat::fill(__m128 _v)
+{
+    int size = (int)total();
+    float* ptr = (float*)data;
+    for (int i = 0; i < size; i++)
+    {
+        _mm_storeu_ps(ptr, _v);
+        ptr += 4;
+    }
+}
 NCNN_FORCEINLINE void Mat::fill(__m128i _v)
 {
     int size = (int)total();
@@ -1091,7 +1052,7 @@ NCNN_FORCEINLINE void Mat::fill(__m128i _v)
         ptr += 8;
     }
 }
-#endif // __AVX__
+#endif // __SSE2__
 
 #if __mips_msa
 NCNN_FORCEINLINE void Mat::fill(v4f32 _v)
